//
// Solver.cpp
//

// Engine includes.
#include "EventStep.h"
#include "GameManager.h"
#include "LogManager.h"
#include "WorldManager.h"
#include "utility.h"

// Game includes.
#include "Path.h"
#include "Solver.h"

// Constructor.
Solver::Solver(stack<Cell> in_path) {

  // Base object.
  setType("Solver");
  path = in_path;
  df::Vector p(path.top().getCol(), path.top().getRow());
  setPosition(p);
  setSolidness(df::SPECTRAL);

  // Draw on top.
  setAltitude(df::MAX_ALTITUDE);

  // Get step events.
  if (registerInterest(df::STEP_EVENT) == -1) {
    df::LogManager &log_manager = df::LogManager::getInstance();
    log_manager.writeLog("Solver::Solver(): registerInterest(STEP_EVENT) failed!");
  }

  // Begin moving.
  countdown = 0;
}

// Handle event.
// Return 0 if ignored, else 1.
int Solver::eventHandler(const df::Event *e) {

  // Check if step event.
  if (e->getType() == df::STEP_EVENT) {

    // Check if time to move.
    countdown--;
    if (countdown > 0)
      return 1;
    countdown = COUNTDOWN_STEP;
    move();
    return 1;
  }

  // If we get here, we have ignored this event.
  return 0;
}

// Move through maze to end.
void Solver::move() {

  // Continue until path is empty.
  if (!path.empty()) { 
    int row = path.top().getRow();
    int col = path.top().getCol();
    path.pop();

    // Move solver to new position and place path.
    df::Vector p(col*COL_WIDTH+1, row*ROW_WIDTH+1);
    new Path(p);
    return;
  }

  // Path empty, so done.
  df::GameManager &game_manager = df::GameManager::getInstance();
  game_manager.setGameOver();

  // Remove all path objects so just see maze at end.
  df::WorldManager &world_manager = df::WorldManager::getInstance();
  df::ObjectList obj = world_manager.getAllObjects();
  df::ObjectListIterator i(&obj);
  for (i.first(); !i.isDone(); i.next()) {
    df::Object *p_o = i.currentObject();
    if (p_o -> getType() == "Path")
      world_manager.markForDelete(p_o);
  }

  return;
}
